/* Common */
html, body {height: 100%;}
body {font-family: Arial;}
ul {list-style: none; padding: 0px;}

/* Icons */
.caption i.fa {width: 25px;}
.caret {margin: 4px 0px 8px 8px}

/* Helpers */
.hidden {display: none;}
.relative {position: relative;}
.fixed {position: fixed;}
.stretched {width: 100%;}
.no-indent {margin: 0px; padding: 0px;}

/* Forms */
.form-control {border-radius: 5px;}
.form-control[readonly] {background-color: inherit; cursor: context-menu;}
.form-group {position: relative;}
form.condensed {max-width: 500px; padding: 0px 30px; margin: auto;}
form .actions {text-align: center;}
form .action button {width: 60%; margin-top: 10px;}

/* Lists */
.list-panel {padding-top: 5px;}
.list-panel input {margin: 10px 0px;}
.list-panel button {width: 100%; margin-bottom: 10px;}
.list-panel .panel-title .qs-clear:hover {font-weight: bold;}
.pager li span {margin: 10px;}

/* Tables */
td {vertical-align: middle !important;}
td .form-group {margin: 0;}
@media screen and (min-width: 992px) {tr {height: 40px;}}
